@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
- * Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+ * Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,9 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#define _BSD_SOURCE
+
+
+
 
 #include "config.h"
 
@@ -29,7 +31,7 @@
 #include <errno.h>
 
 #if defined(HAVE_STRING_H)
-# include <string.h>
+# include <string.h> /* FreeBSD 12.1 */
 #endif
 
 #if defined(HAVE_STRINGS_H)
@@ -38,9 +40,10 @@
 
 #include "cw.h"
 #include "libcw.h"
+#include "libcw_debug.h"
 
 #include "i18n.h"
-#include "cmdline.h"
+#include "cw_cmdline.h"
 #include "cw_copyright.h"
 
 
@@ -59,18 +62,19 @@ static void write_to_cw_sender (const char *format, ...)
     __attribute__ ((__format__ (__printf__, 1, 2)));
 #endif
 
-
+static void signal_handler(int signal_number);
+static void cw_atexit(void);
 
 static cw_config_t *config = NULL; /* program-specific configuration */
 static bool generator = false;     /* have we created a generator? */
-static const char *all_options = "s:|system,d:|device,"
-	"w:|wpm,t:|tone,v:|volume,"
-	"g:|gap,k:|weighting,"
-	"f:|infile,"
-	"e|noecho,m|nomessages,c|nocommands,o|nocombinations,p|nocomments,"
-	"h|help,V|version";
+static bool g_is_running = false;
+
+
+
+
+extern cw_debug_t cw_debug_object;
+
 
-static void cw_atexit(void);
 
 
 /*---------------------------------------------------------------------*/
@@ -164,7 +168,7 @@ parse_stream_query (FILE *stream)
 {
   int c, value;
 
-  c = toupper (fgetc (stream));
+  c = toupper (getc_unlocked (stream));
   switch (c)
     {
     case EOF:
@@ -221,7 +225,7 @@ parse_stream_cwquery (FILE *stream)
   int c, value;
   const char *format;
 
-  c = toupper (fgetc (stream));
+  c = toupper (getc_unlocked (stream));
   switch (c)
     {
     case EOF:
@@ -379,7 +383,7 @@ parse_stream_command (FILE *stream)
 {
   int c;
 
-  c = toupper (fgetc (stream));
+  c = toupper (getc_unlocked (stream));
   switch (c)
     {
     case EOF:
@@ -479,8 +483,12 @@ parse_stream (FILE *stream)
    * Cycle round states depending on input characters.  Comments may be
    * nested inside combinations, but not the other way around; that is,
    * combination starts and ends are not special within comments.
+   *
+   * We need an _unlocked variant of fgetc because otherwise attempt to close
+   * stdin in signal handler (to signal termination of the loop) won't be
+   * possible: fclose() will just hang because stdin will be locked.
    */
-  for (c = fgetc (stream); !feof (stream); c = fgetc (stream))
+  for (c = getc_unlocked (stream); g_is_running && !feof (stream); c = getc_unlocked (stream))
     {
       switch (state)
         {
@@ -532,7 +540,7 @@ parse_stream (FILE *stream)
                */
               int lookahead;
 
-              lookahead = fgetc (stream);
+              lookahead = getc_unlocked (stream);
               ungetc (lookahead, stream);
               send_cw_character (c, lookahead != CW_COMBINATION_END);
             }
@@ -556,6 +564,24 @@ parse_stream (FILE *stream)
 
 
 
+static void signal_handler(int signal_number)
+{
+	fprintf(stderr, _("\nCaught signal %d, exiting...\n"), signal_number);
+
+	/* Stop the main 'read char from stdin and process it'
+	   loop. Ultimately this will lead to exiting of program. */
+	g_is_running = false;
+
+	/* This is needed because if initially there are no characters
+	   available to consume by fgetc() in parse_stream(), the
+	   'for' loop in that function will be stuck at initial
+	   fgetc(). We need to force attempt at first iteration of the
+	   'for' loop. */
+	fclose(stdin);
+}
+
+
+
 
 /**
    \brief Parse command line args, then produce CW output until end of file
@@ -565,23 +591,30 @@ parse_stream (FILE *stream)
 */
 int main (int argc, char *const argv[])
 {
+	//cw_debug_set_flags(&cw_debug_object, CW_DEBUG_MASK);
+	//cw_debug_object.level = CW_DEBUG_INFO;
+
 	atexit(cw_atexit);
 
 	/* Set locale and message catalogs. */
 	i18n_initialize();
 
 	/* Parse combined environment and command line arguments. */
-	int combined_argc;
-	char **combined_argv;
+	int combined_argc = 0;
+	char **combined_argv = NULL;
 	combine_arguments("CW_OPTIONS", argc, argv, &combined_argc, &combined_argv);
 
 	config = cw_config_new(cw_program_basename(argv[0]));
 	if (!config) {
 		return EXIT_FAILURE;
 	}
-	config->is_cw = 1;
+	config->has_feature_sound_system = true;
+	config->has_feature_generator = true;
+	config->has_feature_dot_dash_params = true;
+	config->has_feature_infile = true;
+	config->has_feature_cw_specific = true;
 
-	if (!cw_process_argv(combined_argc, combined_argv, all_options, config)) {
+	if (CW_SUCCESS != cw_process_program_arguments(combined_argc, combined_argv, config)) {
 		fprintf(stderr, _("%s: failed to parse command line args\n"), config->program_name);
 		return EXIT_FAILURE;
 	}
@@ -598,10 +631,10 @@ int main (int argc, char *const argv[])
 		}
 	}
 
-	if (config->audio_system == CW_AUDIO_ALSA
+	if (config->gen_conf.sound_system == CW_AUDIO_ALSA
 	    && cw_is_pa_possible(NULL)) {
 
-		fprintf(stdout, "Selected audio system is ALSA, but audio on your system is handled by PulseAudio. Expect problems with timing.\n");
+		fprintf(stdout, "Selected sound system is ALSA, but sound on your system is handled by PulseAudio. Expect problems with timing.\n");
 		fprintf(stdout, "In this situation it is recommended to run %s like this:\n", config->program_name);
 		fprintf(stdout, "%s -s p\n\n", config->program_name);
 		fprintf(stdout, "Press Enter key to continue\n");
@@ -617,7 +650,7 @@ int main (int argc, char *const argv[])
 	/* Set up signal handlers to exit on a range of signals. */
 	static const int SIGNALS[] = { SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM, 0 };
 	for (int i = 0; SIGNALS[i]; i++) {
-		if (!cw_register_signal_handler(SIGNALS[i], SIG_DFL)) {
+		if (!cw_register_signal_handler(SIGNALS[i], signal_handler)) {
 			fprintf(stderr, _("%s: can't register signal: %s\n"), config->program_name, strerror(errno));
 			return EXIT_FAILURE;
 		}
@@ -626,6 +659,7 @@ int main (int argc, char *const argv[])
 	/* Start producing sine wave (amplitude of the wave will be
 	   zero as long as there are no characters to process). */
 	cw_generator_start();
+	g_is_running = true;
 
 	/* Send stdin stream to CW parsing. */
 	parse_stream(stdin);
