@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
- * Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+ * Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +41,7 @@
 
 #include "libcw.h"
 #include "i18n.h"
-#include "cmdline.h"
+#include "cw_cmdline.h"
 #include "cw_copyright.h"
 #include "dictionary.h"
 #include "memory.h"
@@ -72,13 +72,6 @@ static bool is_sending_active = false;
 
 static cw_config_t *config = NULL; /* program-specific configuration */
 static bool generator = false;     /* have we created a generator? */
-static const char *all_options = "s:|system,d:|device,"
-	"w:|wpm,t:|tone,v:|volume,"
-	"g:|gap,k:|weighting,"
-	"f:|infile,F:|outfile,"
-	"T:|time,"
-	/* "c:|colours,c:|colors,m|mono," */
-	"h|help,V|version";
 
 
 
@@ -134,7 +127,7 @@ typedef struct mode_s *moderef_t;
    The program is always in one of these modes, indicated by
    current_mode. */
 static moderef_t modes = NULL,
-                 current_mode = NULL;
+                 g_current_mode = NULL;
 static int modes_count = 0;
 
 static int queue_get_length(void);
@@ -531,22 +524,22 @@ void queue_transfer_character_to_libcw(void)
 	   empty.  If in keyboard mode, just dequeue anything
 	   currently on the character queue. */
 
-	if (current_mode->type == M_DICTIONARY) {
+	if (g_current_mode->type == M_DICTIONARY) {
 		if (timer_is_expired()) {
 			state_change_to_idle();
 			return;
 		}
 
 		if (queue_get_length() == 0) {
-			queue_enqueue_random_dictionary_text(current_mode, beginning_of_buffer);
+			queue_enqueue_random_dictionary_text(g_current_mode, beginning_of_buffer);
 			if (beginning_of_buffer) {
 				beginning_of_buffer = false;
 			}
 		}
 	}
 
-	if (current_mode->type == M_DICTIONARY
-	    || current_mode->type == M_KEYBOARD) {
+	if (g_current_mode->type == M_DICTIONARY
+	    || g_current_mode->type == M_KEYBOARD) {
 
 		queue_dequeue_character();
 	}
@@ -717,7 +710,7 @@ void mode_initialize(void)
 	memset(modes + count, 0, sizeof (*modes));
 
 	/* Initialize the current mode to be the first listed, and set count. */
-	current_mode = modes;
+	g_current_mode = modes;
 	modes_count = count;
 
 	return;
@@ -761,7 +754,7 @@ int mode_get_count(void)
 */
 int mode_get_current(void)
 {
-	return current_mode - modes;
+	return g_current_mode - modes;
 }
 
 
@@ -785,7 +778,7 @@ const char *mode_get_description(int index)
 */
 bool mode_current_is_type(mode_type_t type)
 {
-	return current_mode->type == type;
+	return g_current_mode->type == type;
 }
 
 
@@ -802,9 +795,9 @@ bool mode_current_is_type(mode_type_t type)
 */
 bool mode_change_to_next(void)
 {
-	current_mode++;
-	if (!current_mode->description) {
-		current_mode--;
+	g_current_mode++;
+	if (!g_current_mode->description) {
+		g_current_mode--;
 		return false;
 	} else {
 		return true;
@@ -825,8 +818,8 @@ bool mode_change_to_next(void)
 */
 bool mode_change_to_previous(void)
 {
-	if (current_mode > modes) {
-		current_mode--;
+	if (g_current_mode > modes) {
+		g_current_mode--;
 		return true;
 	} else {
 		return false;
@@ -854,14 +847,14 @@ void state_change_to_active(void)
 
 	ui_display_state(_("Sending(F9 or Esc to exit)"));
 
-	if (current_mode != last_mode) {
+	if (g_current_mode != last_mode) {
 		ui_clear_main_window();
 		timer_start();
 
 		/* Don't allow a space at the beginning of buffer. */
 		beginning_of_buffer = true;
 
-		last_mode = current_mode;
+		last_mode = g_current_mode;
         }
 
 	ui_refresh_main_window();
@@ -925,9 +918,9 @@ bool mode_is_sending_active(void)
  * character limit imposed by ISO C89 on string literal lengths.
  */
 static const char *const INTRODUCTION = N_(
-  "UNIX/Linux Morse Tutor v3.5.1\n"
+  "UNIX/Linux Morse Tutor v3.6.0\n"
   "Copyright (C) 1997-2006 Simon Baldwin\n"
-  "Copyright (C) 2011-2017 Kamil Ignacak\n"
+  "Copyright (C) 2011-2021 Kamil Ignacak\n"
   "---------------------------------------------------------\n"
   "Cwcp is an interactive Morse code tutor program, designed\n"
   "both for learning Morse code for the first time, and for\n"
@@ -963,7 +956,7 @@ enum
 };
 
 /* User interface event loop running flag. */
-static bool is_running = true;
+static bool g_is_running = true;
 
 /* Color definitions. */
 static const short color_array[] = {
@@ -1408,7 +1401,7 @@ static int interface_interpret(int c)
 	case PSEUDO_KEYF9:
 	case '\n':
 		if (mode_current_is_type(M_EXIT)) {
-			is_running = false;
+			g_is_running = false;
 		} else {
 			if (!mode_is_sending_active()) {
 				state_change_to_active();
@@ -1437,7 +1430,7 @@ static int interface_interpret(int c)
 	case 'C' - CTRL_OFFSET:
 		queue_discard_contents();
 		cw_flush_tone_queue();
-		is_running = false;
+		g_is_running = false;
 		break;
 
 	case KEY_RESIZE:
@@ -1667,12 +1660,8 @@ void ui_update_mode_selection(int old_mode, int current_mode)
 */
 void signal_handler(int signal_number)
 {
-	/* Attempt to wrestle the screen back from curses. */
-	ui_destroy();
-
-	/* Show the signal caught, and exit. */
 	fprintf(stderr, _("\nCaught signal %d, exiting...\n"), signal_number);
-	exit(EXIT_SUCCESS);
+	g_is_running = false;
 }
 
 
@@ -1691,8 +1680,8 @@ int main(int argc, char **argv)
 	i18n_initialize();
 
 	/* Parse combined environment and command line arguments. */
-	int combined_argc;
-	char **combined_argv;
+	int combined_argc = 0;
+	char **combined_argv = NULL;
 
 	/* Parse combined environment and command line arguments. */
 	combine_arguments(_("CWCP_OPTIONS"), argc, argv, &combined_argc, &combined_argv);
@@ -1701,10 +1690,14 @@ int main(int argc, char **argv)
 	if (!config) {
 		return EXIT_FAILURE;
 	}
-	config->has_practice_time = true;
-	config->has_outfile = true;
+	config->has_feature_sound_system = true;
+	config->has_feature_generator = true;
+	config->has_feature_dot_dash_params = true;
+	config->has_feature_practice_time = true;
+	config->has_feature_infile = true;
+	config->has_feature_outfile = true;
 
-	if (!cw_process_argv(combined_argc, combined_argv, all_options, config)) {
+	if (CW_SUCCESS != cw_process_program_arguments(combined_argc, combined_argv, config)) {
 		fprintf(stderr, _("%s: failed to parse command line args\n"), config->program_name);
 		return EXIT_FAILURE;
 	}
@@ -1732,10 +1725,10 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (config->audio_system == CW_AUDIO_ALSA
+	if (config->gen_conf.sound_system == CW_AUDIO_ALSA
 	    && cw_is_pa_possible(NULL)) {
 
-		fprintf(stdout, "Selected audio system is ALSA, but audio on your system is handled by PulseAudio. Expect problems with timing.\n");
+		fprintf(stdout, "Selected sound system is ALSA, but sound on your system is handled by PulseAudio. Expect problems with timing.\n");
 		fprintf(stdout, "In this situation it is recommended to run %s like this:\n", config->program_name);
 		fprintf(stdout, "%s -s p\n\n", config->program_name);
 		fprintf(stdout, "Press Enter key to continue\n");
@@ -1770,7 +1763,7 @@ int main(int argc, char **argv)
 	   speed needs a 10ms (10,000usec) timeout. */
 	ui_initialize();
 	cw_generator_start();
-	while (is_running) {
+	while (g_is_running) {
 		ui_poll_user_input(fileno(stdin), 10000);
 		ui_handle_event(getch());
 	}
