@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
- * Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+ * Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +36,7 @@
 #endif
 
 #include "i18n.h"
-#include "cmdline.h"
+#include "cw_cmdline.h"
 #include "cw_copyright.h"
 #include "memory.h"
 
@@ -67,7 +67,7 @@ struct cwgen_config {
 	uint64_t n_chars_max;  /* Maximal number of characters (excluding spaces) to generate in whole set of groups; may be zero - no limit. */
 
 	char *charset;         /* Set of chars to be used to generate groups. */
-} config = {
+} g_config = {
 	.program_name   = (char *) NULL,
 
 	.n_groups       = INITIAL_GROUPS,
@@ -304,7 +304,6 @@ void cwgen_parse_command_line(int argc, char **argv, struct cwgen_config *config
 				fprintf(stderr, _("%s: invalid limit value: %s\n"), config->program_name, argument);
 				exit(EXIT_FAILURE);
 			}
-			fprintf(stderr, _("%s: valid limit value: %s\n"), config->program_name, argument);
 			break;
 
 		case 'c':
@@ -322,7 +321,7 @@ void cwgen_parse_command_line(int argc, char **argv, struct cwgen_config *config
 
 		case 'h':
 			cwgen_print_help(config->program_name);
-
+			/* Fallthrough. */
 		case 'v':
 			printf(_("%s version %s\n%s\n"),
 			       config->program_name, PACKAGE_VERSION, _(CW_COPYRIGHT));
@@ -355,8 +354,8 @@ void cwgen_parse_command_line(int argc, char **argv, struct cwgen_config *config
 */
 int main(int argc, char **argv)
 {
-	int combined_argc;
-	char **combined_argv;
+	int combined_argc = 0;
+	char **combined_argv = NULL;
 
 	/* Set locale and message catalogs. */
 	i18n_initialize();
@@ -364,21 +363,21 @@ int main(int argc, char **argv)
 	/* Parse combined environment and command line arguments. */
 	combine_arguments(_("CWGEN_OPTIONS"),
 			  argc, argv, &combined_argc, &combined_argv);
-	cwgen_parse_command_line(combined_argc, combined_argv, &config);
+	cwgen_parse_command_line(combined_argc, combined_argv, &g_config);
 
-	if (!config.charset) {
-		config.charset = strdup(DEFAULT_CHARSET);
-		if (!config.charset) {
+	if (!g_config.charset) {
+		g_config.charset = strdup(DEFAULT_CHARSET);
+		if (!g_config.charset) {
 			fprintf(stderr, _("%s: failed to allocate memory\n"), argv[0]);
 			return EXIT_FAILURE;
 		}
 	}
 
 	/* Generate the character groups as requested. */
-	cwgen_generate_characters(&config);
+	cwgen_generate_characters(&g_config);
 	putchar('\n');
 
-	cwgen_free_config(&config);
+	cwgen_free_config(&g_config);
 
 	return EXIT_SUCCESS;
 }
