@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
- * Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+ * Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,31 +19,29 @@
 
 
 
+
 /**
-   \file libcw_debug.c
+   @file libcw_debug.c
 
-   \brief An attempt to create some smart debugging facility to libcw
-   and applications using the library.
+   @brief Debugging facility to libcw and applications using the library.
 */
 
 
 
 
-
 #include "config.h"
 
 
-#define _BSD_SOURCE   /* usleep() */
-//#define _POSIX_SOURCE /* sigaction() */
-//#define _POSIX_C_SOURCE 200112L /* pthread_sigmask() */
 
 
-#include <stdio.h>
-#include <sys/time.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
 #include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/time.h>
+#include <unistd.h>
+
+
 
 
 #include "libcw.h"
@@ -54,11 +52,15 @@
 
 
 
+#define MSG_PREFIX "libcw/debug: "
 
-struct {
+
+
+
+static struct {
 	int flag;
-	const char *message;
-} cw_debug_event_strings[] = {
+	const char * message;
+} event_logging_strings[] = {
 	{ CW_DEBUG_EVENT_TONE_LOW,        "CW_DEBUG_EVENT_TONE_LOW"        },
 	{ CW_DEBUG_EVENT_TONE_MID,        "CW_DEBUG_EVENT_TONE_MID"        },
 	{ CW_DEBUG_EVENT_TONE_HIGH,       "CW_DEBUG_EVENT_TONE_HIGH"       },
@@ -71,18 +73,17 @@ struct {
 
 
 
-
 /* Human-readable labels for debug levels.
    Other modules can access the table only through pointer in debug
    object. I don't expose this table (by making it globally visible)
    to decrease number of 'extern' declarations. */
-static const char *cw_debug_level_labels[] = { "DD", "II", "WW", "EE" };
+static const char * cw_debug_level_labels[] = { "[DD]", "[II]", "[WW]", "[EE]" };
 
 
 
 
 cw_debug_t cw_debug_object = {
-	.flags = CW_DEBUG_STDLIB | CW_DEBUG_SOUND_SYSTEM, //CW_DEBUG_TONE_QUEUE; //CW_DEBUG_KEYER_STATES | CW_DEBUG_STRAIGHT_KEY | CW_DEBUG_KEYING; // | CW_DEBUG_TONE_QUEUE;,
+	.flags = CW_DEBUG_STDLIB | CW_DEBUG_SOUND_SYSTEM,
 	.n = 0,
 	.n_max = 1,
 	.level = CW_DEBUG_NONE,
@@ -107,38 +108,45 @@ cw_debug_t cw_debug_object_ev = {
 
 
 
-static void cw_debug_flush(cw_debug_t *debug_object);
 
+static void cw_event_debugging_flush_internal(cw_debug_t * debug_object) __attribute__((unused));
 
 
 
 
 /**
-   \brief Write all events from the debug object to a file
+   @brief Write all events from the debug object to a file
 
-   Function writes all events stored in the \p debug object to file
+   Function writes all events stored in the @p debug_object to file
    associated with the object, and removes the events.
 
    List of events is preceded with "FLUSH START\n" line, and
    followed by "FLUSH END\n" line.
 
-   \param debug - debug object
+   Function is used only in advanced debugging of events in library.
+
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @param[in] debug_object debug object from which to flush events
 */
-void cw_debug_flush(cw_debug_t *debug)
+void cw_event_debugging_flush_internal(cw_debug_t * debug_object)
 {
-	if (debug->n <= 0) {
+	if (debug_object->n <= 0) {
 		return;
 	}
 
-	long long int diff = debug->events[debug->n - 1].sec - debug->events[0].sec;
-	diff = debug->events[debug->n - 1].sec - diff - 1;
+	long long int diff = debug_object->events[debug_object->n - 1].sec - debug_object->events[0].sec;
+	diff = debug_object->events[debug_object->n - 1].sec - diff - 1;
 
 	fprintf(stderr, "FLUSH START\n");
-	for (int i = 0; i < debug->n; i++) {
+	for (int i = 0; i < debug_object->n; i++) {
 		fprintf(stderr, "libcwevent:\t%06lld%06lld\t%s\n",
-			debug->events[i].sec - diff, debug->events[i].usec,
-			cw_debug_event_strings[debug->events[i].event].message);
+			debug_object->events[i].sec - diff, debug_object->events[i].usec,
+			event_logging_strings[debug_object->events[i].event].message);
 	}
+	debug_object->n = 0;
 	fprintf(stderr, "FLUSH END\n");
 
 	fflush(stderr);
@@ -149,16 +157,19 @@ void cw_debug_flush(cw_debug_t *debug)
 
 
 
-
 /**
-   \brief Set a value of internal debug flags variable
+   @brief Set a value of internal debug flags variable
 
    Assign specified value to library's internal debug flags variable.
    Note that this function doesn't *append* given flag to the variable,
    it erases existing value and assigns new one. Use cw_get_debug_flags()
    if you want to OR new flag with existing ones.
 
-   \param new_value - new value to be assigned to the library
+   @internal
+   @reviewed 2020-10-31
+   @endinternal
+
+   @param flags new value of flags to be assigned to the debug object
 */
 void cw_set_debug_flags(uint32_t flags)
 {
@@ -169,11 +180,24 @@ void cw_set_debug_flags(uint32_t flags)
 
 
 
-
 /**
-   testedin::test_cw_debug_flags()
+   @brief Set a value of debug flags in given debug variable
+
+   Assign specified value to given debug variable.
+
+   Note that this function doesn't *append* given flag to the
+   variable, it erases existing value and assigns new one. Use
+   cw_debug_get_flags() if you want to OR new flag with existing ones.
+
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @param[in] debug_object debug object for which to set flags
+   @param[in] flags new value to be assigned to the object, a sum of one or
+   more of CW_DEBUG_* flags from libcw.h
 */
-void cw_debug_set_flags(cw_debug_t *debug_object, uint32_t flags)
+void cw_debug_set_flags(cw_debug_t * debug_object, uint32_t flags)
 {
 	debug_object->flags = flags;
 	return;
@@ -182,13 +206,19 @@ void cw_debug_set_flags(cw_debug_t *debug_object, uint32_t flags)
 
 
 
-
 /**
-   \brief Get current library's debug flags
+   @brief Get library's current debug flags
 
-   Function returns value of library's internal debug variable.
+   Function returns value of library's internal debug variable, a sum of one
+   or more of CW_DEBUG_* flags from libcw.h.
 
-   \return value of library's debug flags variable
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @internal TODO: there is some initialization code using LIBCW_DEBUG env. Perhaps we should include it somehow in libcw2 code. @endinternal
+
+   @return value of library's debug flags variable
 */
 uint32_t cw_get_debug_flags(void)
 {
@@ -205,7 +235,7 @@ uint32_t cw_get_debug_flags(void)
 			 * Set the debug flags from LIBCW_DEBUG.  If it is an invalid
 			 * numeric, treat it as 0; there is no error checking.
 			 */
-			const char *debug_value = getenv("LIBCW_DEBUG");
+			const char * debug_value = getenv("LIBCW_DEBUG");
 			if (debug_value) {
 				cw_debug_object.flags = strtoul(debug_value, NULL, 0);
 			}
@@ -220,11 +250,19 @@ uint32_t cw_get_debug_flags(void)
 
 
 
-
 /**
-   testedin::test_cw_debug_flags()
+   @brief Get current debug flags from given debug object
+
+   Function returns value of debug object's debug flags, a sum of one or more
+   of CW_DEBUG_* flags from libcw.h.
+
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @return value of debug object's debug flags
 */
-uint32_t cw_debug_get_flags(cw_debug_t *debug_object)
+uint32_t cw_debug_get_flags(const cw_debug_t * debug_object)
 {
 	/* FIXME: what about initialization? */
 	return debug_object->flags;
@@ -233,19 +271,22 @@ uint32_t cw_debug_get_flags(cw_debug_t *debug_object)
 
 
 
-
 /**
-   \brief Check if given debug flag is set
+   @brief Check if given debug flag is set
 
-   Function checks if a specified debug flag is set in internal
-   variable of libcw library.
+   Function checks if a specified debug flag is set in given debug object.
 
-   \param flag - flag to be checked.
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
 
-   \return true if given flag is set
-   \return false if given flag is not set
+   @param[in] debug_object debug object to be checked
+   @param[in] flag flag to be checked, one of CW_DEBUG_* flags from libcw.h
+
+   @return true if given flag is set
+   @return false otherwise
 */
-bool cw_debug_has_flag(cw_debug_t *debug_object, uint32_t flag)
+bool cw_debug_has_flag(const cw_debug_t * debug_object, uint32_t flag)
 {
 	if (debug_object) {
 		return debug_object->flags & flag;
@@ -257,25 +298,37 @@ bool cw_debug_has_flag(cw_debug_t *debug_object, uint32_t flag)
 
 
 
-
 #ifdef LIBCW_WITH_DEV
 
 
 
 
+/**
+   @brief Print configuration of generator
 
-void cw_dev_debug_print_generator_setup(cw_gen_t *gen)
+   Print to stderr some of generator's parameters.
+
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @param[in] gen generator
+*/
+void cw_dev_debug_print_generator_setup_internal(const cw_gen_t * gen)
 {
-	fprintf(stderr, "audio system:         %s\n",     cw_get_audio_system_label(gen->audio_system));
-	if (gen->audio_system == CW_AUDIO_OSS) {
+	fprintf(stderr, "sound system:         %s\n",     cw_get_audio_system_label(gen->sound_system));
+#ifdef LIBCW_WITH_OSS
+	if (gen->sound_system == CW_AUDIO_OSS) {
 		fprintf(stderr, "OSS version           %X.%X.%X\n",
-			gen->oss_version.x, gen->oss_version.y, gen->oss_version.z);
+			gen->oss_data.version.x, gen->oss_data.version.y, gen->oss_data.version.z);
 	}
-	fprintf(stderr, "audio device:         \"%s\"\n",  gen->audio_device);
-	fprintf(stderr, "sample rate:          %d Hz\n",  gen->sample_rate);
+#endif
+	fprintf(stderr, "picked sound device:  \"%s\"\n",  gen->picked_device_name);
+	fprintf(stderr, "sample rate:          %u Hz\n",  gen->sample_rate);
 
-#ifdef LIBCW_WITH_PULSEAUDIO
-	if (gen->audio_system == CW_AUDIO_PA) {
+	/* Temporarily disabled because gen->pa_data.ba is missing. */
+#if 0 // def LIBCW_WITH_PULSEAUDIO
+	if (gen->sound_system == CW_AUDIO_PA) {
 		fprintf(stderr, "PulseAudio latency:   %llu us\n", (unsigned long long int) gen->pa_data.latency_usecs);
 
 		if (gen->pa_data.ba.prebuf == (uint32_t) -1) {
@@ -316,7 +369,7 @@ void cw_dev_debug_print_generator_setup(cw_gen_t *gen)
 	fprintf(stderr, "send speed:           %d wpm\n", gen->send_speed);
 	fprintf(stderr, "volume:               %d %%\n",  gen->volume_percent);
 	fprintf(stderr, "frequency:            %d Hz\n",  gen->frequency);
-	fprintf(stderr, "audio buffer size:    %d\n",     gen->buffer_n_samples);
+	fprintf(stderr, "sound buffer size:    %d\n",     gen->buffer_n_samples);
 
 	fprintf(stderr, "debug sink file:      %s\n", gen->dev_raw_sink != -1 ? "yes" : "no");
 
@@ -326,12 +379,28 @@ void cw_dev_debug_print_generator_setup(cw_gen_t *gen)
 
 
 
+/**
+   @brief Write generator's samples to debug file
 
-int cw_dev_debug_raw_sink_write_internal(cw_gen_t *gen)
+   This function does any actual writing only for generators
+   configured to use OSS, Alsa and PulseAudio sound sinks. Using the
+   function on generators configured with other sound sinks doesn't
+   produce any output and the function always returns CW_SUCCESS.
+
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @param[in] gen generator
+
+   @return CW_SUCCESS on write success
+   @return CW_FAILURE otherwise
+*/
+int cw_dev_debug_raw_sink_write_internal(cw_gen_t * gen)
 {
-	if (gen->audio_system == CW_AUDIO_NONE
-	    || gen->audio_system == CW_AUDIO_NULL
-	    || gen->audio_system == CW_AUDIO_CONSOLE) {
+	if (gen->sound_system != CW_AUDIO_OSS
+	    && gen->sound_system != CW_AUDIO_ALSA
+	    && gen->sound_system != CW_AUDIO_PA) {
 
 		return CW_SUCCESS;
 	}
@@ -347,12 +416,12 @@ int cw_dev_debug_raw_sink_write_internal(cw_gen_t *gen)
 		gen->buffer[samples - 1] = 0x8000;
 #endif
 
-		int n_bytes = sizeof (gen->buffer[0]) * gen->buffer_n_samples;
+		const size_t n_bytes = sizeof (gen->buffer[0]) * gen->buffer_n_samples;
 
 		int rv = write(gen->dev_raw_sink, gen->buffer, n_bytes);
 		if (rv == -1) {
-			cw_debug_msg ((&cw_debug_object_dev), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-				      "libcw_debug: write error: %s (gen->dev_raw_sink = %ld, gen->buffer = %ld, n_bytes = %d)", strerror(errno), (long) gen->dev_raw_sink, (long) gen->buffer, n_bytes);
+			cw_debug_msg (&cw_debug_object_dev, CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
+				      MSG_PREFIX "write error: %s (gen->dev_raw_sink = %ld, gen->buffer = %ld, n_bytes = %zu)", strerror(errno), (long) gen->dev_raw_sink, (long) gen->buffer, n_bytes);
 			return CW_FAILURE;
 		}
 	}
@@ -363,10 +432,24 @@ int cw_dev_debug_raw_sink_write_internal(cw_gen_t *gen)
 
 
 
+/**
+   @brief Add debug event to debug object
 
-void cw_debug_event_internal(cw_debug_t *debug_object, uint32_t flag, uint32_t event, const char *func, int line)
+   Function is used only in advanced debugging of events in library.
+
+   @internal
+   @reviewed 2020-08-01
+   @endinternal
+
+   @param[in] debug_object debug object
+   @param[in] flag one of CW_DEBUG_* flags from libcw.h
+   @param[in] event one of CW_DEBUG_EVENT_* flags from libcw_debug.h
+   @param[in] func function where the event occurred
+   @param[in] line file line in which the event occurred
+*/
+void cw_debug_event_internal(cw_debug_t * debug_object, uint32_t flag, uint32_t event, const char * func __attribute__((unused)), int line __attribute__((unused)))
 {
-	if (!debug_object) {
+	if (NULL == debug_object) {
 		return;
 	}
 
@@ -384,8 +467,7 @@ void cw_debug_event_internal(cw_debug_t *debug_object, uint32_t flag, uint32_t e
 	debug_object->n++;
 
 	if (debug_object->n >= debug_object->n_max) {
-		cw_debug_flush(debug_object);
-		debug_object->n = 0;
+		cw_event_debugging_flush_internal(debug_object);
 	}
 
 	return;
@@ -394,61 +476,4 @@ void cw_debug_event_internal(cw_debug_t *debug_object, uint32_t flag, uint32_t e
 
 
 
-
 #endif /* #ifdef LIBCW_WITH_DEV */
-
-
-
-
-
-/* Unit tests. */
-
-#ifdef LIBCW_UNIT_TESTS
-
-
-
-
-
-#include <inttypes.h>
-
-#include "libcw_test.h"
-
-
-
-
-
-/**
-   \brief Test getting and setting of debug flags.
-
-   tests::cw_debug_set_flags()
-   tests::cw_debug_get_flags()
-*/
-unsigned int test_cw_debug_flags_internal(void)
-{
-	/* Store current flags for period of tests. */
-	uint32_t flags_backup = cw_debug_get_flags(&cw_debug_object);
-
-	for (uint32_t i = 1; i <= CW_DEBUG_MASK; i++) {
-		cw_debug_set_flags(&cw_debug_object, i);
-
-		cw_assert ((&cw_debug_object)->flags & i,
-			   "Failed to set debug flag %"PRIu32"\n", i);
-
-		cw_assert (cw_debug_get_flags(&cw_debug_object) == i,
-			   "Failed to get debug flag %"PRIu32"\n", i);
-	}
-
-	int n = printf("libcw/debug: cw_debug_set/get_flags():");
-	CW_TEST_PRINT_TEST_RESULT (false, n);
-
-	/* Restore original flags. */
-	cw_debug_set_flags(&cw_debug_object, flags_backup);
-
-	return 0;
-}
-
-
-
-
-
-#endif /* #ifdef LIBCW_UNIT_TESTS */
