@@ -1,6 +1,6 @@
 /*
   This file is a part of unixcw project.
-  unixcw project is covered by GNU General Public License.
+  unixcw project is covered by GNU General Public License, version 2 or later.
 */
 
 #ifndef H_LIBCW_PA
@@ -9,24 +9,20 @@
 
 
 
-
 #include "config.h"
 
 
 
 
-
 #ifdef LIBCW_WITH_PULSEAUDIO
 
-#include <pulse/simple.h>
 #include <pulse/error.h>
+#include <pulse/simple.h>
 
 typedef struct cw_pa_data_struct {
-	pa_simple *s;       /* audio handle */
-	pa_sample_spec ss;  /* sample specification */
+	pa_simple * simple;    /* Audio handle. */
+	pa_sample_spec spec;   /* Sample specification. */
 	pa_usec_t latency_usecs;
-
-	pa_buffer_attr ba;
 } cw_pa_data_t;
 
 #endif /* #ifdef LIBCW_WITH_PULSEAUDIO */
@@ -34,13 +30,14 @@ typedef struct cw_pa_data_struct {
 
 
 
-
 #include "libcw_gen.h"
 
-int cw_pa_configure(cw_gen_t *gen, const char *device);
 
 
 
+cw_ret_t cw_pa_init_gen_internal(cw_gen_t * gen);
+
+
 
 
 #endif /* #ifndef H_LIBCW_PA */
