@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-  Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+  Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -32,19 +32,21 @@
 
 
 
-
 #include "config.h"
 
 
-#include <signal.h>
+
 #include <errno.h>
+#include <signal.h>
 #include <stdlib.h>
 
 
+
+
 #include "libcw.h"
-#include "libcw_signal.h"
 #include "libcw_debug.h"
 #include "libcw_gen.h"
+#include "libcw_signal.h"
 #include "libcw_utils.h"
 
 
@@ -59,7 +61,6 @@
 
 
 
-
 /* http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=403043 */
 #if defined(NSIG)             /* Debian GNU/Linux: signal.h; Debian kFreeBSD: signal.h (libc0.1-dev_2.13-21_kfreebsd-i386.deb) */
 #define CW_SIG_MAX (NSIG)
@@ -76,6 +77,10 @@
 
 
 
+#define MSG_PREFIX "libcw: "
+
+
+
 
 extern cw_debug_t cw_debug_object;
 extern cw_debug_t cw_debug_object_ev;
@@ -135,7 +140,7 @@ void cw_sigalrm_handlers_caller_internal(__attribute__((unused)) int signal_numb
 	     handler < CW_SIGALRM_HANDLERS_MAX && cw_sigalrm_handlers[handler]; handler++) {
 
 		cw_debug_msg ((&cw_debug_object_dev), CW_DEBUG_INTERNAL, CW_DEBUG_DEBUG,
-			      "libcw: SIGALRM handler #%d", handler);
+			      MSG_PREFIX "SIGALRM handler #%d", handler);
 
 		(cw_sigalrm_handlers[handler])();
 	}
@@ -171,7 +176,7 @@ int cw_timer_run_internal(int usecs)
 	int status = setitimer(ITIMER_REAL, &itimer, NULL);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: setitimer(%d): %s", usecs, strerror(errno));
+			      MSG_PREFIX "setitimer(%d): %s", usecs, strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -208,7 +213,7 @@ int cw_timer_run_with_handler_internal(int usecs, void (*sigalrm_handler)(void))
 	   of the lower level SIGALRM handler to the list of known
 	   handlers. */
 	if (sigalrm_handler) {
-		int handler;
+		int handler = 0;
 
 		/* Search for this handler, or the first free entry,
 		   stopping at the last entry in the table even if it's
@@ -228,7 +233,7 @@ int cw_timer_run_with_handler_internal(int usecs, void (*sigalrm_handler)(void))
 			if (cw_sigalrm_handlers[handler]) {
 				errno = ENOMEM;
 				cw_debug_msg ((&cw_debug_object), CW_DEBUG_INTERNAL, CW_DEBUG_ERROR,
-					      "libcw: overflow cw_sigalrm_handlers");
+					      MSG_PREFIX "overflow cw_sigalrm_handlers");
 				return CW_FAILURE;
 			} else {
 				cw_sigalrm_handlers[handler] = sigalrm_handler;
@@ -248,7 +253,7 @@ int cw_timer_run_with_handler_internal(int usecs, void (*sigalrm_handler)(void))
 		if (pthread_kill(cw_generator->thread.id, SIGALRM) != 0) {
 	        // if (raise(SIGALRM) != 0) {
 			cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-				      "libcw: raise()");
+				      MSG_PREFIX "raise()");
 			return CW_FAILURE;
 		}
 
@@ -283,7 +288,7 @@ int cw_sigalrm_install_top_level_handler_internal(void)
 		int status = sigaction(SIGALRM, &action, &cw_sigalrm_original_disposition);
 		if (status == -1) {
 			cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-				      "libcw: sigaction(): %s", strerror(errno));
+				      MSG_PREFIX "sigaction(): %s", strerror(errno));
 			return CW_FAILURE;
 		}
 
@@ -317,7 +322,7 @@ int cw_sigalrm_restore_internal(void)
 		/* Put back the SIGALRM information saved earlier. */
 		int status = sigaction(SIGALRM, &cw_sigalrm_original_disposition, NULL);
 		if (status == -1) {
-			perror ("libcw: sigaction");
+			perror (MSG_PREFIX "sigaction");
 			return CW_FAILURE;
 		}
 
@@ -345,13 +350,14 @@ int cw_sigalrm_restore_internal(void)
 */
 bool cw_sigalrm_is_blocked_internal(void)
 {
-	sigset_t empty_set, current_set;
+	sigset_t empty_set;
+	sigset_t current_set;
 
 	/* Prepare empty set of signals */
 	int status = sigemptyset(&empty_set);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: sigemptyset(): %s", strerror(errno));
+			      MSG_PREFIX "sigemptyset(): %s", strerror(errno));
 		return true;
 	}
 
@@ -359,7 +365,7 @@ bool cw_sigalrm_is_blocked_internal(void)
 	status = sigprocmask(SIG_BLOCK, &empty_set, &current_set);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: sigprocmask(): %s", strerror(errno));
+			      MSG_PREFIX "sigprocmask(): %s", strerror(errno));
 		return true;
 	}
 
@@ -396,7 +402,7 @@ int cw_sigalrm_block_internal(bool block)
 	int status = sigemptyset(&set);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: sigemptyset(): %s", strerror(errno));
+			      MSG_PREFIX "sigemptyset(): %s", strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -404,7 +410,7 @@ int cw_sigalrm_block_internal(bool block)
 	status = sigaddset(&set, SIGALRM);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: sigaddset(): %s", strerror(errno));
+			      MSG_PREFIX "sigaddset(): %s", strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -412,7 +418,7 @@ int cw_sigalrm_block_internal(bool block)
 	status = pthread_sigmask(block ? SIG_BLOCK : SIG_UNBLOCK, &set, NULL);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: pthread_sigmask(): %s", strerror(errno));
+			      MSG_PREFIX "pthread_sigmask(): %s", strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -457,13 +463,14 @@ void cw_block_callback(int block)
 */
 int cw_signal_wait_internal(void)
 {
-	sigset_t empty_set, current_set;
+	sigset_t empty_set;
+	sigset_t current_set;
 
 	/* Prepare empty set of signals */
 	int status = sigemptyset(&empty_set);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: sigemptyset(): %s", strerror(errno));
+			      MSG_PREFIX "sigemptyset(): %s", strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -471,7 +478,7 @@ int cw_signal_wait_internal(void)
 	status = sigprocmask(SIG_BLOCK, &empty_set, &current_set);
 	if (status == -1) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: sigprocmask(): %s", strerror(errno));
+			      MSG_PREFIX "sigprocmask(): %s", strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -479,7 +486,7 @@ int cw_signal_wait_internal(void)
 	status = sigsuspend(&current_set);
 	if (status == -1 && errno != EINTR) {
 		cw_debug_msg ((&cw_debug_object), CW_DEBUG_STDLIB, CW_DEBUG_ERROR,
-			      "libcw: suspend(): %s", strerror(errno));
+			      MSG_PREFIX "suspend(): %s", strerror(errno));
 		return CW_FAILURE;
 	}
 
@@ -500,33 +507,39 @@ static void (*cw_signal_callbacks[CW_SIG_MAX])(int);
 
 
 /**
-   \brief Generic function calling signal handlers
+   @brief Generic function calling signal handlers
 
-   Signal handler function registered when cw_register_signal_handler()
-   is called.
-   The function resets the library (with cw_complete_reset()), and then,
-   depending on value of signal handler for given \p signal_number:
-   \li calls exit(EXIT_FAILURE) if signal handler is SIG_DFL, or
-   \li continues without further actions if signal handler is SIG_IGN, or
-   \li calls the signal handler.
+   Depending on value of signal handler for given @p signal_number the
+   function calls the registered signal handler.
 
-   The signal handler for given \p signal_number is either a pre-set, default
-   value, or is a value registered earlier with cw_register_signal_handler().
+   The signal handler for given @p signal_number is a value registered
+   earlier with cw_register_signal_handler().
 
-   \param signal_number
+   @internal
+   @reviewed 2020-11-11
+   @endinternal
+
+   @param[in] signal_number number of signal to handle
 */
 void cw_signal_main_handler_internal(int signal_number)
 {
 	cw_debug_msg ((&cw_debug_object), CW_DEBUG_FINALIZATION, CW_DEBUG_INFO,
-		      "libcw: caught signal %d", signal_number);
+		      MSG_PREFIX "caught signal %d", signal_number);
 
-	/* Reset the library and retrieve the signal's handler. */
-	cw_complete_reset();
+	/* Don't call this function in signal handler context. It will
+	   lead to calling pthread_cond_*() functions, and that will
+	   lead to program hanging. */
+	//cw_complete_reset();
+	
 	void (*callback_func)(int) = cw_signal_callbacks[signal_number];
 
-	/* The default action is to stop the process; exit(1) seems to cover it. */
 	if (callback_func == SIG_DFL) {
-		exit(EXIT_FAILURE);
+		/* Calling exit() would lead to calls of functions
+		   registered with atexit(), which would be executed
+		   in signal handler context. Those functions called
+		   by atexit() could call pthread_cond_*() functions,
+		   and that would lead to program hanging. */
+		//exit(EXIT_FAILURE);
 	} else if (callback_func == SIG_IGN) {
 		/* continue */
 	} else {
@@ -587,13 +600,14 @@ int cw_register_signal_handler(int signal_number, void (*callback_func)(int))
 	}
 
 	/* Install our handler as the actual handler. */
-	struct sigaction action, original_disposition;
+	struct sigaction action;
+	struct sigaction original_disposition;
 	action.sa_handler = cw_signal_main_handler_internal;
 	action.sa_flags = SA_RESTART;
 	sigemptyset(&action.sa_mask);
 	int status = sigaction(signal_number, &action, &original_disposition);
 	if (status == -1) {
-		perror("libcw: sigaction");
+		perror(MSG_PREFIX "sigaction");
 		return CW_FAILURE;
 	}
 
@@ -604,7 +618,7 @@ int cw_register_signal_handler(int signal_number, void (*callback_func)(int))
 
 		status = sigaction(signal_number, &original_disposition, NULL);
 		if (status == -1) {
-			perror("libcw: sigaction");
+			perror(MSG_PREFIX "sigaction");
 			return CW_FAILURE;
 		}
 
@@ -648,7 +662,7 @@ int cw_unregister_signal_handler(int signal_number)
 	struct sigaction original_disposition;
 	int status = sigaction(signal_number, NULL, &original_disposition);
 	if (status == -1) {
-		perror("libcw: sigaction");
+		perror(MSG_PREFIX "sigaction");
 		return CW_FAILURE;
 	}
 
@@ -665,7 +679,7 @@ int cw_unregister_signal_handler(int signal_number)
 	sigemptyset(&action.sa_mask);
 	status = sigaction(signal_number, &action, NULL);
 	if (status == -1) {
-		perror("libcw: sigaction");
+		perror(MSG_PREFIX "sigaction");
 		return CW_FAILURE;
 	}
 
