@@ -1,5 +1,5 @@
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-// Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+// Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -306,6 +306,9 @@ void Application::start()
 
 	sender->clear();
 	receiver->clear();
+#ifdef XCWCP_WITH_REC_TEST
+	receiver->start_test_code();
+#endif
 
 	/* Accessing proper action through this->startstop should also
 	   work. */
@@ -348,6 +351,9 @@ void Application::stop()
 	poll_timer->stop();
 	sender->clear();
 	receiver->clear();
+#ifdef XCWCP_WITH_REC_TEST
+	receiver->stop_test_code();
+#endif
 
 	/* Saving speed for restore on next start. */
 	saved_receive_speed = cw_get_receive_speed();
@@ -421,7 +427,7 @@ void Application::sync_speed()
 			cw_disable_adaptive_receive();
 			if (!cw_set_receive_speed(speed_spin->value())) {
 				perror("cw_set_receive_speed");
-				abort();
+				return;
 			}
 			cw_enable_adaptive_receive();
 		}
@@ -446,12 +452,12 @@ void Application::change_speed()
 	if (is_using_libcw) {
 		if (!cw_set_send_speed(speed_spin->value())) {
 			perror("cw_set_send_speed");
-			abort();
+			return;
 		}
 		if (!cw_get_adaptive_receive_state()) {
 			if (!cw_set_receive_speed(speed_spin->value())) {
 				perror("cw_set_receive_speed");
-				abort();
+				return;
 			}
 		}
 	}
@@ -475,7 +481,7 @@ void Application::change_frequency()
 	if (is_using_libcw) {
 		if (!cw_set_frequency(frequency_spin->value())) {
 			perror("cw_set_frequency");
-			abort();
+			return;
 		}
 	}
 
@@ -498,7 +504,7 @@ void Application::change_volume()
 	if (is_using_libcw) {
 		if (!cw_set_volume(volume_spin->value())) {
 			perror("cw_set_volume");
-			abort();
+			return;
 		}
 	}
 
@@ -521,7 +527,7 @@ void Application::change_gap()
 	if (is_using_libcw) {
 		if (!cw_set_gap(gap_spin->value())) {
 			perror("cw_set_gap");
-			abort();
+			return;
 		}
 	}
 
@@ -602,7 +608,7 @@ void Application::change_adaptive_receive()
 			cw_disable_adaptive_receive();
 			if (!cw_set_receive_speed(saved_receive_speed)) {
 				perror("cw_set_receive_speed");
-				abort();
+				return;
 			}
 			cw_enable_adaptive_receive();
 		} else {
@@ -613,7 +619,7 @@ void Application::change_adaptive_receive()
 			cw_disable_adaptive_receive();
 			if (!cw_set_receive_speed(speed_spin->value())) {
 				perror("cw_set_receive_speed");
-				abort();
+				return;
 			}
 		}
 	}
@@ -909,10 +915,10 @@ void Application::make_mode_combo()
 	/* Append each mode represented in the modes set to the combo
 	   box's contents, then synchronize the current mode. */
 	for (int index = 0; index < modeset.get_count(); index++) {
-		const QVariant data(index);
+		const QVariant mode_data(index);
 		const Mode *mode = modeset.get(index);
 		const QString string = QString::fromUtf8(mode->get_description().c_str());
-		mode_combo->addItem(string, data);
+		mode_combo->addItem(string, mode_data);
 	}
 	modeset.set_current(mode_combo->currentIndex());
 
@@ -977,49 +983,49 @@ void Application::make_program_menu(void)
 
 void Application::make_settings_menu(void)
 {
-	QMenu *settings = new QMenu(_("&Settings"), this);
-	QMainWindow::menuBar()->addMenu(settings);
+	QMenu *my_settings_menu = new QMenu(_("&Settings"), this);
+	QMainWindow::menuBar()->addMenu(my_settings_menu);
 
 
 	reverse_paddles_action = new QAction(_("&Reverse Paddles"), this);
 	reverse_paddles_action->setCheckable(true);
 	reverse_paddles_action->setChecked(false);
-	settings->addAction(reverse_paddles_action);
+	my_settings_menu->addAction(reverse_paddles_action);
 
 
 	curtis_mode_b_action = new QAction(_("&Curtis Mode B Timing"), this);
 	curtis_mode_b_action->setCheckable(true);
 	curtis_mode_b_action->setChecked(false);
 	connect(curtis_mode_b_action, SIGNAL (toggled(bool)), SLOT (change_curtis_mode_b()));
-	settings->addAction(curtis_mode_b_action);
+	my_settings_menu->addAction(curtis_mode_b_action);
 
 
 	adaptive_receive_action = new QAction(_("&Adaptive CW Receive Speed"), this);
 	adaptive_receive_action->setCheckable(true);
 	adaptive_receive_action->setChecked(true);
 	connect(adaptive_receive_action, SIGNAL (toggled(bool)), SLOT (change_adaptive_receive()));
-	settings->addAction(adaptive_receive_action);
+	my_settings_menu->addAction(adaptive_receive_action);
 
 
-	settings->addSeparator();
+	my_settings_menu->addSeparator();
 
 
 	font_settings_action = new QAction(_("&Text font..."), this);
 	connect(font_settings_action, SIGNAL (triggered(bool)), SLOT (fonts()));
-	settings->addAction(font_settings_action);
+	my_settings_menu->addAction(font_settings_action);
 
 
 	color_settings_action = new QAction(_("&Text color..."), this);
 	connect(color_settings_action, SIGNAL (triggered(bool)), SLOT (colors()));
-	settings->addAction(color_settings_action);
+	my_settings_menu->addAction(color_settings_action);
 
 
-	settings->addSeparator();
+	my_settings_menu->addSeparator();
 
 
 	toolbar_visibility_action = new QAction(_("Hide toolbar"), this);
 	connect(toolbar_visibility_action, SIGNAL (triggered(bool)), SLOT (toggle_toolbar()));
-	settings->addAction(toolbar_visibility_action);
+	my_settings_menu->addAction(toolbar_visibility_action);
 
 	return;
 }
@@ -1100,17 +1106,17 @@ void Application::make_auxiliaries_end(void)
 	   paddles are pressed, but (since it doesn't receive timing
 	   parameters) it won't be able to identify entered Morse
 	   code. */
-	cw_register_keying_callback(libcw_keying_event_static, &(receiver->timer));
+	cw_register_keying_callback(libcw_keying_event_static, &receiver->main_timer);
 
-	/* The call above registered receiver->timer as a generic
+	/* The call above registered receiver->main_timer as a generic
 	   argument to a callback. However, libcw needs to know when
 	   the argument happens to be of type 'struct timeval'. This
 	   is why we have this second call, explicitly passing
 	   receiver's timer to libcw. */
-	cw_iambic_keyer_register_timer(&(receiver->timer));
+	cw_iambic_keyer_register_timer(&receiver->main_timer);
 
-	gettimeofday(&(receiver->timer), NULL);
-	//fprintf(stderr, "time on aux config: %10ld : %10ld\n", receiver->timer.tv_sec, receiver->timer.tv_usec);
+	gettimeofday(&(receiver->main_timer), NULL);
+	//fprintf(stderr, "time on aux config: %10ld : %10ld\n", receiver->main_timer.tv_sec, receiver->main_timer.tv_usec);
 
 	QString label("Output: ");
 	label += cw_generator_get_audio_system_label();
@@ -1124,13 +1130,13 @@ void Application::make_auxiliaries_end(void)
 
 
 
-void Application::check_audio_system(cw_config_t *config)
+void Application::check_sound_system(cw_config_t *config)
 {
-	if (config->audio_system == CW_AUDIO_ALSA
+	if (config->gen_conf.sound_system == CW_AUDIO_ALSA
 	    && cw_is_pa_possible(NULL)) {
 
 		QMessageBox msgBox;
-		QString message1 = _("Selected audio system is ALSA, but audio on your system is handled by PulseAudio.");
+		QString message1 = _("Selected sound system is ALSA, but sound on your system is handled by PulseAudio.");
 		QString message2 = _("Expect various problems.\n");
 		QString message3 = _("In this situation it is recommended to run %1 like this:\n" \
 				     "%2 -s p\n\n");
