@@ -1,5 +1,5 @@
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-// Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+// Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -38,30 +38,21 @@
 #include "application.h"
 
 #include "libcw.h"
-//#include "libcw_debug.h"
+#include "libcw_debug.h"
 
 #include "i18n.h"
-#include "cmdline.h"
+#include "cw_cmdline.h"
 #include "cw_copyright.h"
 #include "dictionary.h"
 
 
-//extern cw_debug_t cw_debug_object;
+extern cw_debug_t cw_debug_object;
 
 void xcwcp_atexit(void);
 
 namespace {
 cw_config_t *config = NULL; /* program-specific configuration */
 bool generator = false;     /* have we created a generator? */
-std::string all_options = "s:|sound,d:|device,"
-	"w:|wpm,t:|tone,v:|volume,"
-	"g:|gap,k:|weighting,"
-	// "i:|infile,F:|outfile,"
-	// "T:|time,"
-	"h|help,V|version";
-
-
-
 
 
 
@@ -93,8 +84,8 @@ int main(int argc, char **argv)
 {
 	try {
 
-		//cw_debug_set_flags(&cw_debug_object, CW_DEBUG_KEYING | CW_DEBUG_GENERATOR | CW_DEBUG_TONE_QUEUE | CW_DEBUG_RECEIVE_STATES | CW_DEBUG_KEYER_STATES | CW_DEBUG_INTERNAL| CW_DEBUG_PARAMETERS);
-		//cw_debug_object.level = CW_DEBUG_DEBUG;
+		//cw_debug_set_flags(&cw_debug_object, CW_DEBUG_MASK); //KEYING | CW_DEBUG_GENERATOR | CW_DEBUG_TONE_QUEUE | CW_DEBUG_RECEIVE_STATES | CW_DEBUG_KEYER_STATES | CW_DEBUG_INTERNAL| CW_DEBUG_PARAMETERS);
+		//cw_debug_object.level = CW_DEBUG_WARNING;
 
 		atexit(xcwcp_atexit);
 
@@ -102,8 +93,8 @@ int main(int argc, char **argv)
 		i18n_initialize();
 
 		/* Parse combined environment and command line arguments. */
-		int combined_argc;
-		char **combined_argv;
+		int combined_argc = 0;
+		char **combined_argv = NULL;
 
 
 		// Parse combined environment and command line arguments.  Arguments
@@ -117,10 +108,11 @@ int main(int argc, char **argv)
 		if (!config) {
 			return EXIT_FAILURE;
 		}
-		config->has_practice_time = 0;
-		config->has_infile = false;
+		config->has_feature_sound_system = true;
+		config->has_feature_generator = true;
+		config->has_feature_dot_dash_params = true;
 
-		if (!cw_process_argv(argc, argv, all_options.c_str(), config)) {
+		if (CW_SUCCESS != cw_process_program_arguments(argc, argv, config)) {
 			fprintf(stderr, _("%s: failed to parse command line args\n"), config->program_name);
 			return EXIT_FAILURE;
 		}
@@ -173,7 +165,7 @@ int main(int argc, char **argv)
 		q_application.connect(&q_application, SIGNAL (lastWindowClosed ()),
 				      &q_application, SLOT (quit ()));
 
-		application->check_audio_system(config);
+		application->check_sound_system(config);
 		// Enter the application event loop.
 		int rv = q_application.exec();
 
