@@ -1,5 +1,5 @@
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-// Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+// Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -35,6 +35,18 @@
 #include "i18n.h"
 
 
+#ifdef XCWCP_WITH_REC_TEST
+#include <cctype>
+#include <unistd.h>
+
+#include <cw_rec_tester.h>
+
+#include "libcw2.h"
+#include "../libcw/libcw_key.h"
+#include "../libcw/libcw_gen.h"
+#include "../libcw/libcw_tq.h"
+#include "../libcw/libcw_utils.h"
+#endif
 
 
 
@@ -206,8 +218,8 @@ void Receiver::sk_event(bool is_down)
 	   how straight key does not. Apparently the timer is used to
 	   recognize and distinguish dots from dashes. Maybe straight
 	   key could have such timer as well? */
-	gettimeofday(&timer, NULL);
-	//fprintf(stderr, "time on Skey down:  %10ld : %10ld\n", timer.tv_sec, timer.tv_usec);
+	gettimeofday(&this->main_timer, NULL);
+	//fprintf(stderr, "time on Skey down:  %10ld : %10ld\n", this->main_timer.tv_sec, this->main_timer.tv_usec);
 
 	cw_notify_straight_key_event(is_down);
 
@@ -239,8 +251,8 @@ void Receiver::ik_left_event(bool is_down, bool is_reverse_paddles)
 
 		   TODO: why libcw can't create such timestamp for
 		   first event for us? */
-		gettimeofday(&timer, NULL);
-		//fprintf(stderr, "time on Lkey down:  %10ld : %10ld\n", timer.tv_sec, timer.tv_usec);
+		gettimeofday(&this->main_timer, NULL);
+		//fprintf(stderr, "time on Lkey down:  %10ld : %10ld\n", this->main_timer.tv_sec, this->main_timer.tv_usec);
 	}
 
 	/* Inform libcw about state of left paddle regardless of state
@@ -274,8 +286,8 @@ void Receiver::ik_right_event(bool is_down, bool is_reverse_paddles)
 		   In case of iambic keyer the timestamps for every
 		   next (non-initial) "paddle up" or "paddle down"
 		   event in a character will be created by libcw. */
-		gettimeofday(&timer, NULL);
-		//fprintf(stderr, "time on Rkey down:  %10ld : %10ld\n", timer.tv_sec, timer.tv_usec);
+		gettimeofday(&this->main_timer, NULL);
+		//fprintf(stderr, "time on Rkey down:  %10ld : %10ld\n", this->main_timer.tv_sec, this->main_timer.tv_usec);
 	}
 
 	/* Inform libcw about state of left paddle regardless of state
@@ -353,7 +365,7 @@ void Receiver::handle_libcw_keying_event(struct timeval *t, int key_state)
 		//fprintf(stderr, "start receive tone: %10ld . %10ld\n", t->tv_sec, t->tv_usec);
 		if (!cw_start_receive_tone(t)) {
 			perror("cw_start_receive_tone");
-			abort();
+			return;
 		}
 	} else {
 		/* Key up. */
@@ -369,16 +381,16 @@ void Receiver::handle_libcw_keying_event(struct timeval *t, int key_state)
 				   was shorter than noise threshold).
 				   No problem, not an error. */
 				break;
-
 			case ENOMEM:
+			case ERANGE:
+			case EINVAL:
 			case ENOENT:
 				libcw_receive_errno = errno;
 				cw_clear_receive_buffer();
 				break;
-
 			default:
 				perror("cw_end_receive_tone");
-				abort();
+				return;
 			}
 		}
 	}
@@ -413,9 +425,24 @@ void Receiver::clear()
 void Receiver::poll_report_error()
 {
 	/* Handle any receive errors detected on tone end but delayed until here. */
-	app->show_status(libcw_receive_errno == ENOENT
-			 ? _("Badly formed CW element")
-			 : _("Receive buffer overrun"));
+
+	switch (libcw_receive_errno) {
+	case ENOMEM:
+		app->show_status(_("Representation buffer too small"));
+		break;
+	case ERANGE:
+		app->show_status(_("Internal error"));
+		break;
+	case EINVAL:
+		app->show_status(_("Internal timestamp error"));
+		break;
+	case ENOENT:
+		app->show_status(_("Badly formed CW element"));
+		break;
+	default:
+		app->show_status(_("Internal problem"));
+		break;
+	}
 
 	libcw_receive_errno = 0;
 
@@ -431,23 +458,63 @@ void Receiver::poll_report_error()
 */
 void Receiver::poll_character()
 {
-	char c;
+	char c = 0;
 
-	/* Don't use receiver.timer - it is used exclusively for
+	/* Don't use receiver.main_timer - it is used exclusively for
 	   marking initial "key down" events. Use local throw-away
 	   timer2.
 
-	   Additionally using reveiver.timer here would mess up time
-	   intervals measured by receiver.timer, and that would
+	   Additionally using reveiver.main_timer here would mess up time
+	   intervals measured by receiver.main_timer, and that would
 	   interfere with recognizing dots and dashes. */
 	struct timeval timer2;
 	gettimeofday(&timer2, NULL);
 	//fprintf(stderr, "poll_receive_char:  %10ld : %10ld\n", timer2.tv_sec, timer2.tv_usec);
-	if (cw_receive_character(&timer2, &c, NULL, NULL)) {
+
+	bool is_end_of_word;
+	if (cw_receive_character(&timer2, &c, &is_end_of_word, NULL)) {
 		/* Receiver stores full, well formed
 		   character. Display it. */
 		textarea->append(c);
 
+#ifdef XCWCP_WITH_REC_TEST
+		fprintf(stderr, "[II] Character: '%c'\n", c);
+
+		this->rec_tester.received_string[this->rec_tester.received_string_i++] = c;
+
+		bool is_end_of_word_r = false;
+		bool is_error_r = false;
+		char representation[20] = { 0 };
+		int cw_ret = cw_receive_representation(&timer2, representation, &is_end_of_word_r, &is_error_r);
+		if (CW_SUCCESS != cw_ret) {
+			fprintf(stderr, "[EE] Character: failed to get representation\n");
+			exit(EXIT_FAILURE);
+		}
+
+		if (is_end_of_word_r != is_end_of_word) {
+			fprintf(stderr, "[EE] Character: 'is end of word' markers mismatch: %d != %d\n", is_end_of_word_r, is_end_of_word);
+			exit(EXIT_FAILURE);
+		}
+
+		if (is_end_of_word_r) {
+			fprintf(stderr, "[EE] Character: 'is end of word' marker is unexpectedly 'true'\n");
+			exit(EXIT_FAILURE);
+		}
+
+		const char looked_up = cw_representation_to_character(representation);
+		if (0 == looked_up) {
+			fprintf(stderr, "[EE] Character: Failed to look up character for representation\n");
+			exit(EXIT_FAILURE);
+		}
+
+		if (looked_up != c) {
+			fprintf(stderr, "[EE] Character: Looked up character is different than received: %c != %c\n", looked_up, c);
+		}
+
+		fprintf(stderr, "[II] Character: Representation: %c -> '%s'\n", c, representation);
+#endif
+
+
 		/* A full character has been received. Directly after
 		   it comes a space. Either a short inter-character
 		   space followed by another character (in this case
@@ -483,16 +550,22 @@ void Receiver::poll_character()
 			   character (yet). Try harder. */
 			break;
 
-		case ENOENT:
-			/* Invalid character in receiver's buffer. */
+		case ENOENT: /* Invalid character in receiver's buffer. */
 			cw_clear_receive_buffer();
 			textarea->append('?');
 			app->show_status(QString(_("Unknown character received at %1 WPM")).arg(cw_get_receive_speed()));
 			break;
 
+		case EINVAL:
+			/* Timestamp error. */
+			cw_clear_receive_buffer();
+			textarea->append('?');
+			app->show_status(QString(_("Internal error")));
+			break;
+
 		default:
 			perror("cw_receive_character");
-			abort();
+			return;
 		}
 	}
 
@@ -520,16 +593,57 @@ void Receiver::poll_space()
 	   will treat it as inter-word space, and communicate it over
 	   is_end_of_word.
 
-	   Don't use receiver.timer - it is used eclusively for
+	   Don't use receiver.main_timer - it is used eclusively for
 	   marking initial "key down" events. Use local throw-away
 	   timer2. */
 	struct timeval timer2;
 	gettimeofday(&timer2, NULL);
 	//fprintf(stderr, "poll_space(): %10ld : %10ld\n", timer2.tv_sec, timer2.tv_usec);
-	cw_receive_character(&timer2, NULL, &is_end_of_word, NULL);
+
+	char c = 0;
+	cw_receive_character(&timer2, &c, &is_end_of_word, NULL);
 	if (is_end_of_word) {
-		//fprintf(stderr, "End of word\n\n");
+		//fprintf(stderr, "End of word '%c'\n\n", c);
 		textarea->append(' ');
+
+#ifdef XCWCP_WITH_REC_TEST
+		fprintf(stderr, "[II] Space:\n");
+
+		/* cw_receive_character() will return through 'c'
+		   variable the last character that was polled before
+		   space.
+
+		   Maybe this is good, maybe this is bad, but this is
+		   the legacy behaviour that we will keep
+		   supporting. */
+		if (' ' == c) {
+			fprintf(stderr, "[EE] Space: returned character should not be space\n");
+			exit(EXIT_FAILURE);
+		}
+
+
+		this->rec_tester.received_string[this->rec_tester.received_string_i++] = ' ';
+
+		bool is_end_of_word_r = false;
+		bool is_error_r = false;
+		char representation[20] = { 0 };
+		int cw_ret = cw_receive_representation(&timer2, representation, &is_end_of_word_r, &is_error_r);
+		if (CW_SUCCESS != cw_ret) {
+			fprintf(stderr, "[EE] Space: Failed to get representation\n");
+			exit(EXIT_FAILURE);
+		}
+
+		if (is_end_of_word_r != is_end_of_word) {
+			fprintf(stderr, "[EE] Space: 'is end of word' markers mismatch: %d != %d\n", is_end_of_word_r, is_end_of_word);
+			exit(EXIT_FAILURE);
+		}
+
+		if (!is_end_of_word_r) {
+			fprintf(stderr, "[EE] Space: 'is end of word' marker is unexpectedly 'false'\n");
+			exit(EXIT_FAILURE);
+		}
+#endif
+
 		cw_clear_receive_buffer();
 		is_pending_inter_word_space = false;
 	} else {
@@ -549,4 +663,135 @@ void Receiver::poll_space()
 	return;
 }
 
+
+
+
+#ifdef XCWCP_WITH_REC_TEST
+
+
+
+
+void low_tone_queue_callback(void * arg)
+{
+	cw_rec_tester_t * tester = (cw_rec_tester_t *) arg;
+
+	for (int i = 0; i < tester->characters_to_enqueue; i++) {
+		const char c = tester->input_string[tester->input_string_i];
+		if ('\0' == c) {
+			/* Unregister ourselves. */
+			cw_tq_register_low_level_callback_internal(tester->gen->tq, NULL, NULL, 0);
+			break;
+		} else {
+			cw_gen_enqueue_character(tester->gen, c);
+			tester->input_string_i++;
+		}
+	}
+
+	return;
+}
+
+
+
+
+void test_callback_func(void * arg, int key_state)
+{
+	/* Inform xcwcp receiver (which will inform libcw receiver)
+	   about new state of straight key ("sk").
+
+	   libcw receiver will process the new state and we will later
+	   try to poll a character or space from it. */
+
+	Receiver * xcwcp_receiver = (Receiver *) arg;
+	//fprintf(stderr, "Callback function, key state = %d\n", key_state);
+	xcwcp_receiver->sk_event(key_state);
+}
+
+
+
+
+/*
+  Code that generates info about timing of input events for receiver.
+
+  We could generate the info and the events using a big array of
+  timestamps and a call to usleep(), but instead we are using a new
+  generator that can inform us when marks/spaces start.
+*/
+void * receiver_input_generator_fn(void * arg)
+{
+	Receiver * xcwcp_receiver = (Receiver *) arg;
+
+	cw_rec_tester_init(&xcwcp_receiver->rec_tester);
+	cw_rec_tester_init_text_buffers(&xcwcp_receiver->rec_tester, 1);
+
+
+	/* Using Null sound system because this generator is only used
+	   to enqueue text and control key. Sound will be played by
+	   main generator used by xcwcp */
+	cw_gen_config_t gen_conf;
+	memset(&gen_conf, 0, sizeof (gen_conf));
+	gen_conf.sound_system = CW_AUDIO_NULL;
+
+	cw_gen_t * gen = cw_gen_new(&gen_conf);
+	cw_key_t key;
+
+	xcwcp_receiver->rec_tester.gen = gen;
+	cw_tq_register_low_level_callback_internal(xcwcp_receiver->rec_tester.gen->tq, low_tone_queue_callback, &xcwcp_receiver->rec_tester, 5);
+
+	cw_key_register_generator(&key, gen);
+	cw_gen_register_value_tracking_callback_internal(gen, test_callback_func, arg);
+	//cw_key_register_keying_callback(&key, test_callback_func, arg);
+
+	/* Start sending the test string. Registered callback will be
+	   called on every mark/space. Enqueue only initial part of
+	   string, just to start sending, the rest should be sent by
+	   'low watermark' callback. */
+	cw_gen_start(gen);
+	for (int i = 0; i < 5; i++) {
+		const char c = xcwcp_receiver->rec_tester.input_string[xcwcp_receiver->rec_tester.input_string_i];
+		if ('\0' == c) {
+			/* A very short input string. */
+			break;
+		} else {
+			cw_gen_enqueue_character(xcwcp_receiver->rec_tester.gen, c);
+			xcwcp_receiver->rec_tester.input_string_i++;
+		}
+	}
+
+	/* Wait for all characters to be played out. */
+	cw_tq_wait_for_level_internal(gen->tq, 0);
+	cw_usleep_internal(1000 * 1000);
+
+	cw_gen_delete(&gen);
+	xcwcp_receiver->rec_tester.generating_in_progress = false;
+
+	cw_rec_tester_evaluate_receive_correctness(&xcwcp_receiver->rec_tester);
+
+
+	return NULL;
+}
+
+
+
+
+void Receiver::start_test_code()
+{
+	pthread_create(&this->receiver_test_code_thread_id, NULL, receiver_input_generator_fn, this);
+}
+
+
+
+
+void Receiver::stop_test_code()
+{
+	pthread_cancel(this->receiver_test_code_thread_id);
+}
+
+
+
+
+#endif
+
+
+
+
 }  /* namespace cw */
