@@ -1,5 +1,5 @@
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-// Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+// Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -22,6 +22,9 @@
 #include <QMouseEvent>
 #include <QKeyEvent>
 
+#ifdef XCWCP_WITH_REC_TEST
+#include <cw_rec_tester.h>
+#endif
 
 
 
@@ -77,6 +80,14 @@ namespace cw {
 		/* Clear out queued data on stop, mode change, etc. */
 		void clear();
 
+#ifdef XCWCP_WITH_REC_TEST
+		void start_test_code();
+		void stop_test_code();
+		pthread_t receiver_test_code_thread_id;
+
+		cw_rec_tester_t rec_tester;
+#endif
+
 		/* Timer for measuring length of dots and dashes.
 
 		   Initial value of the timestamp is created by
@@ -85,7 +96,7 @@ namespace cw {
 		   on specific time intervals. The intervals are a
 		   function of keyboard key presses or mouse button
 		   presses recorded by xcwcp. */
-		struct timeval timer;
+		struct timeval main_timer;
 
 	private:
 		Application *app;
