@@ -1,5 +1,5 @@
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-// Copyright (C) 2011-2017  Kamil Ignacak (acerion@wp.pl)
+// Copyright (C) 2011-2021  Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -162,8 +162,9 @@ void Sender::dequeue_and_play_character()
 	queue.pop_front();
 	if (!cw_send_character(c)) {
 		perror("cw_send_character");
-		/* TODO: don't call abort(). */
-		abort();
+		QString status = _("Failed to send character '%1'");
+		app->show_status(status.arg(c));
+		return;
 	}
 
 	/* Update the status bar with the character being played.  Put
